//+------------------------------------------------------------------+
//|                                                       EA NBC.mq5 |
//|                                                      Aleksej1966 |
//|                        https://www.mql5.com/ru/users/aleksej1966 |
//+------------------------------------------------------------------+
#property copyright "Aleksej1966"
#property link      "https://www.mql5.com/ru/users/aleksej1966"
#property version   "1.00"
#include <Trade\Trade.mqh>
CTrade trade;

input ushort PeriodSMA=210,
             PeriodEMA=130,
             PeriodSMMA=235,
             PeriodLWMA=140,
             LvlOpen=10,
             LvlClose=65;

input bool Filter=false;

char buy,sell;
int periodSMA,periodEMA,periodSMMA,periodLWMA,period_max;
long coeffsma[],coeffema[],coeffsmma[],coefflwma[];
double lot;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   trade.SetDeviationInPoints(10);
   trade.SetAsyncMode(true);
   trade.SetTypeFillingBySymbol(_Symbol);

   lot=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);

   periodSMA=MathMax(2,PeriodSMA);
   periodEMA=MathMax(2,PeriodEMA);
   periodSMMA=periodSMA>2? MathMax(2,PeriodSMMA):MathMax(3,PeriodSMMA);
   periodLWMA=periodEMA>2? MathMax(2,PeriodLWMA):MathMax(3,PeriodLWMA);

   period_max=MathMax(MathMax(periodSMA,periodEMA),MathMax(periodSMMA,periodLWMA));

   ArrayResize(coeffsma,periodSMA);
   ArrayInitialize(coeffsma,-1);
   coeffsma[0]=periodSMA-1;

   ArrayResize(coeffema,periodEMA);
   ArrayInitialize(coeffema,-1);
   for(int i=periodEMA-2; i>=0; i--)
      coeffema[i]=2*coeffema[i+1];
   coeffema[0]=(long)(coeffema[0]+MathPow(2,periodEMA)-1);

   ArrayResize(coeffsmma,periodSMMA);
   long denom=0;
   for(int i=0; i<(periodSMMA+1)/2; i++)
     {
      coeffsmma[i]=-(i+1);
      coeffsmma[periodSMMA-i-1]=coeffsmma[i];
      denom=denom-2*coeffsmma[i];
     }
   coeffsmma[0]=coeffsmma[0]+denom;

   ArrayResize(coefflwma,periodLWMA);
   for(int i=0; i<periodLWMA; i++)
      coefflwma[i]=i-periodLWMA;
   coefflwma[0]=coefflwma[0]+periodLWMA*(periodLWMA+1)/2;

   int bars=MathMin(iBars(_Symbol,PERIOD_CURRENT),TerminalInfoInteger(TERMINAL_MAXBARS));

   for(int i=bars-period_max-1; i>=0; i--)
      CalcSignal(i);

   NewBar();
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   if(NewBar()==true)
     {
      CalcSignal(0);

      if(buy>0)
         trade.PositionOpen(_Symbol,ORDER_TYPE_BUY,lot,SymbolInfoDouble(_Symbol,SYMBOL_ASK),0,0,NULL);

      if(sell>0)
         trade.PositionOpen(_Symbol,ORDER_TYPE_SELL,lot,SymbolInfoDouble(_Symbol,SYMBOL_BID),0,0,NULL);

      if(buy<0 || sell<0)
        {
         ENUM_POSITION_TYPE type=buy<0?POSITION_TYPE_BUY:POSITION_TYPE_SELL;
         for(int i=PositionsTotal()-1; i>=0; i--)
           {
            ulong ticket=PositionGetTicket(i);
            if(PositionSelectByTicket(ticket)==true && type==(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE))
               trade.PositionClose(ticket,10);
           }
        }
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalcSignal(int bar)
  {
//---
   static long arraySMA[][2],arrayEMA[][2],arraySMMA[][2],arrayLWMA[][2],cnt;

   if(cnt==0)
     {
      ArrayResize(arraySMA,0,100);
      ArrayResize(arrayEMA,0,100);
      ArrayResize(arraySMMA,0,100);
      ArrayResize(arrayLWMA,0,100);
     }

   long sumSMA=0,sumEMA=0,sumSMMA=0,sumLWMA=0;

   for(int i=0; i<period_max; i++)
     {
      int open=(int)MathRound(iOpen(_Symbol,PERIOD_CURRENT,bar+i)/_Point);
      if(i<periodSMA)
         sumSMA=sumSMA+coeffsma[i]*open;
      if(i<periodEMA)
         sumEMA=sumEMA+coeffema[i]*open;
      if(i<periodSMMA)
         sumSMMA=sumSMMA+coeffsmma[i]*open;
      if(i<periodLWMA)
         sumLWMA=sumLWMA+coefflwma[i]*open;
     }

   CalcStatistic(arraySMA,sumSMA);
   CalcStatistic(arrayEMA,sumEMA);
   CalcStatistic(arraySMMA,sumSMMA);
   CalcStatistic(arrayLWMA,sumLWMA);

   cnt++;

   if(bar==0)
     {
      buy=0;
      sell=0;

      long p1=CalcProb(arraySMA,sumSMA),
           p2=CalcProb(arrayEMA,sumEMA),
           p3=CalcProb(arraySMMA,sumSMMA),
           p4=CalcProb(arrayLWMA,sumLWMA);

      double pdn=100.0*p1*p2*p3*p4/(p1*p2*p3*p4+(cnt-p1)*(cnt-p2)*(cnt-p3)*(cnt-p4)),
             pup=100-pdn;

      if(pdn<LvlOpen)
         buy=1;

      if(pup<LvlOpen)
         sell=1;

      if(pdn>=LvlClose)
         buy=-1;

      if(pup>=LvlClose)
         sell=-1;

      if(Filter==true)
        {
         static double last_pdn,last_pup;

         if(buy==1 && last_pdn>pdn)
            buy=0;

         if(sell==1 && last_pup<pup)
            sell=0;

         last_pdn=pdn;
         last_pup=pup;
        }
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
long CalcProb(long &array[][2],long val)
  {
//---
   long prob=0;
   int indx=ArrayBsearch(array,val);

   for(int i=0; i<indx; i++)
      prob=prob+array[i][1];

   return(prob);
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalcStatistic(long &array[][2],long val)
  {
//---
   int ind=ArrayBsearch(array,val);
   if(ind>=0 && array[ind][0]==val)
      array[ind][1]++;
   else
     {
      int size=ArrayRange(array,0);
      ArrayResize(array,size+1);
      array[size][0]=val;
      array[size][1]=1;
      ArraySort(array);
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool NewBar()
  {
//---
   static long lastbar;
   long curbar=SeriesInfoInteger(_Symbol,PERIOD_CURRENT,SERIES_LASTBAR_DATE);
   if(lastbar<curbar)
     {
      lastbar=curbar;
      return(true);
     }
   return(false);
//---
  }
//+------------------------------------------------------------------+
